# Generated by Django 2.2.12 on 2020-05-29 20:16

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import phonenumber_field.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0045_assign_unlock_grouppagepermission"),
        ("wagtaildonate", "0027_thank_you_email_subject_not_mandatory"),
    ]

    operations = [
        migrations.AlterField(
            model_name="donation",
            name="donation_page",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="wagtailcore.Page",
            ),
        ),
        migrations.AlterField(
            model_name="donationexportlog",
            name="donation_type",
            field=models.CharField(
                choices=[
                    ("single", "Single"),
                    ("recurring", "Recurring"),
                    ("pay_in", "Pay in"),
                ],
                max_length=10,
            ),
        ),
        migrations.AlterField(
            model_name="recurringdonation",
            name="donation_page",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="wagtailcore.Page",
            ),
        ),
        migrations.CreateModel(
            name="PayIn",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sms_consent",
                    models.BooleanField(
                        blank=True, default=None, null=True, verbose_name="SMS consent"
                    ),
                ),
                (
                    "phone_consent",
                    models.BooleanField(blank=True, default=None, null=True),
                ),
                (
                    "post_consent",
                    models.BooleanField(blank=True, default=None, null=True),
                ),
                (
                    "email_consent",
                    models.BooleanField(blank=True, default=None, null=True),
                ),
                (
                    "transaction_id",
                    models.CharField(max_length=50, verbose_name="Transaction ID"),
                ),
                (
                    "transaction_status",
                    models.CharField(
                        choices=[
                            ("settling", "Settling"),
                            ("settled", "Settled"),
                            ("failed", "Failed"),
                            ("unknown", "Unknown"),
                        ],
                        max_length=100,
                    ),
                ),
                (
                    "provider_transaction_status",
                    models.CharField(blank=True, max_length=100),
                ),
                ("payment_method", models.CharField(max_length=100)),
                ("amount", models.DecimalField(decimal_places=2, max_digits=10)),
                (
                    "first_name",
                    models.CharField(max_length=100, verbose_name="First name"),
                ),
                ("surname", models.CharField(max_length=100, verbose_name="Surname")),
                (
                    "email",
                    models.EmailField(
                        blank=True, max_length=255, verbose_name="Email address"
                    ),
                ),
                (
                    "phone_number",
                    phonenumber_field.modelfields.PhoneNumberField(
                        blank=True,
                        max_length=128,
                        region=None,
                        verbose_name="Phone number",
                    ),
                ),
                (
                    "address_line_1",
                    models.CharField(max_length=255, verbose_name="Address line 1"),
                ),
                (
                    "address_line_2",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="Address line 2"
                    ),
                ),
                ("town", models.CharField(max_length=255, verbose_name="Town")),
                (
                    "postal_code",
                    models.CharField(max_length=50, verbose_name="Postal code"),
                ),
                (
                    "country",
                    models.CharField(
                        choices=[
                            ("GB", "United Kingdom"),
                            ("IE", "Ireland"),
                            ("AF", "Afghanistan"),
                            ("AL", "Albania"),
                            ("DZ", "Algeria"),
                            ("AS", "American Samoa"),
                            ("AD", "Andorra"),
                            ("AO", "Angola"),
                            ("AI", "Anguilla"),
                            ("AQ", "Antarctica"),
                            ("AG", "Antigua and Barbuda"),
                            ("AR", "Argentina"),
                            ("AM", "Armenia"),
                            ("AW", "Aruba"),
                            ("AU", "Australia"),
                            ("AT", "Austria"),
                            ("AZ", "Azerbaijan"),
                            ("BS", "Bahamas"),
                            ("BH", "Bahrain"),
                            ("BD", "Bangladesh"),
                            ("BB", "Barbados"),
                            ("BY", "Belarus"),
                            ("BE", "Belgium"),
                            ("BZ", "Belize"),
                            ("BJ", "Benin"),
                            ("BM", "Bermuda"),
                            ("BT", "Bhutan"),
                            ("BO", "Bolivia, Plurinational State of"),
                            ("BQ", "Bonaire, Sint Eustatius and Saba"),
                            ("BA", "Bosnia and Herzegovina"),
                            ("BW", "Botswana"),
                            ("BV", "Bouvet Island"),
                            ("BR", "Brazil"),
                            ("IO", "British Indian Ocean Territory"),
                            ("BN", "Brunei Darussalam"),
                            ("BG", "Bulgaria"),
                            ("BF", "Burkina Faso"),
                            ("BI", "Burundi"),
                            ("CV", "Cabo Verde"),
                            ("KH", "Cambodia"),
                            ("CM", "Cameroon"),
                            ("CA", "Canada"),
                            ("KY", "Cayman Islands"),
                            ("CF", "Central African Republic"),
                            ("TD", "Chad"),
                            ("CL", "Chile"),
                            ("CN", "China"),
                            ("CX", "Christmas Island"),
                            ("CC", "Cocos (Keeling) Islands"),
                            ("CO", "Colombia"),
                            ("KM", "Comoros"),
                            ("CG", "Congo"),
                            ("CD", "Congo, The Democratic Republic of the"),
                            ("CK", "Cook Islands"),
                            ("CR", "Costa Rica"),
                            ("HR", "Croatia"),
                            ("CU", "Cuba"),
                            ("CW", "Curaçao"),
                            ("CY", "Cyprus"),
                            ("CZ", "Czechia"),
                            ("CI", "Côte d'Ivoire"),
                            ("DK", "Denmark"),
                            ("DJ", "Djibouti"),
                            ("DM", "Dominica"),
                            ("DO", "Dominican Republic"),
                            ("EC", "Ecuador"),
                            ("EG", "Egypt"),
                            ("SV", "El Salvador"),
                            ("GQ", "Equatorial Guinea"),
                            ("ER", "Eritrea"),
                            ("EE", "Estonia"),
                            ("SZ", "Eswatini"),
                            ("ET", "Ethiopia"),
                            ("FK", "Falkland Islands (Malvinas)"),
                            ("FO", "Faroe Islands"),
                            ("FJ", "Fiji"),
                            ("FI", "Finland"),
                            ("FR", "France"),
                            ("GF", "French Guiana"),
                            ("PF", "French Polynesia"),
                            ("TF", "French Southern Territories"),
                            ("GA", "Gabon"),
                            ("GM", "Gambia"),
                            ("GE", "Georgia"),
                            ("DE", "Germany"),
                            ("GH", "Ghana"),
                            ("GI", "Gibraltar"),
                            ("GR", "Greece"),
                            ("GL", "Greenland"),
                            ("GD", "Grenada"),
                            ("GP", "Guadeloupe"),
                            ("GU", "Guam"),
                            ("GT", "Guatemala"),
                            ("GG", "Guernsey"),
                            ("GN", "Guinea"),
                            ("GW", "Guinea-Bissau"),
                            ("GY", "Guyana"),
                            ("HT", "Haiti"),
                            ("HM", "Heard Island and McDonald Islands"),
                            ("VA", "Holy See (Vatican City State)"),
                            ("HN", "Honduras"),
                            ("HK", "Hong Kong"),
                            ("HU", "Hungary"),
                            ("IS", "Iceland"),
                            ("IN", "India"),
                            ("ID", "Indonesia"),
                            ("IR", "Iran, Islamic Republic of"),
                            ("IQ", "Iraq"),
                            ("IM", "Isle of Man"),
                            ("IL", "Israel"),
                            ("IT", "Italy"),
                            ("JM", "Jamaica"),
                            ("JP", "Japan"),
                            ("JE", "Jersey"),
                            ("JO", "Jordan"),
                            ("KZ", "Kazakhstan"),
                            ("KE", "Kenya"),
                            ("KI", "Kiribati"),
                            ("KP", "Korea, Democratic People's Republic of"),
                            ("KR", "Korea, Republic of"),
                            ("KW", "Kuwait"),
                            ("KG", "Kyrgyzstan"),
                            ("LA", "Lao People's Democratic Republic"),
                            ("LV", "Latvia"),
                            ("LB", "Lebanon"),
                            ("LS", "Lesotho"),
                            ("LR", "Liberia"),
                            ("LY", "Libya"),
                            ("LI", "Liechtenstein"),
                            ("LT", "Lithuania"),
                            ("LU", "Luxembourg"),
                            ("MO", "Macao"),
                            ("MG", "Madagascar"),
                            ("MW", "Malawi"),
                            ("MY", "Malaysia"),
                            ("MV", "Maldives"),
                            ("ML", "Mali"),
                            ("MT", "Malta"),
                            ("MH", "Marshall Islands"),
                            ("MQ", "Martinique"),
                            ("MR", "Mauritania"),
                            ("MU", "Mauritius"),
                            ("YT", "Mayotte"),
                            ("MX", "Mexico"),
                            ("FM", "Micronesia, Federated States of"),
                            ("MD", "Moldova, Republic of"),
                            ("MC", "Monaco"),
                            ("MN", "Mongolia"),
                            ("ME", "Montenegro"),
                            ("MS", "Montserrat"),
                            ("MA", "Morocco"),
                            ("MZ", "Mozambique"),
                            ("MM", "Myanmar"),
                            ("NA", "Namibia"),
                            ("NR", "Nauru"),
                            ("NP", "Nepal"),
                            ("NL", "Netherlands"),
                            ("NC", "New Caledonia"),
                            ("NZ", "New Zealand"),
                            ("NI", "Nicaragua"),
                            ("NE", "Niger"),
                            ("NG", "Nigeria"),
                            ("NU", "Niue"),
                            ("NF", "Norfolk Island"),
                            ("MK", "North Macedonia"),
                            ("MP", "Northern Mariana Islands"),
                            ("NO", "Norway"),
                            ("OM", "Oman"),
                            ("PK", "Pakistan"),
                            ("PW", "Palau"),
                            ("PS", "Palestine, State of"),
                            ("PA", "Panama"),
                            ("PG", "Papua New Guinea"),
                            ("PY", "Paraguay"),
                            ("PE", "Peru"),
                            ("PH", "Philippines"),
                            ("PN", "Pitcairn"),
                            ("PL", "Poland"),
                            ("PT", "Portugal"),
                            ("PR", "Puerto Rico"),
                            ("QA", "Qatar"),
                            ("RO", "Romania"),
                            ("RU", "Russian Federation"),
                            ("RW", "Rwanda"),
                            ("RE", "Réunion"),
                            ("BL", "Saint Barthélemy"),
                            ("SH", "Saint Helena, Ascension and Tristan da Cunha"),
                            ("KN", "Saint Kitts and Nevis"),
                            ("LC", "Saint Lucia"),
                            ("MF", "Saint Martin (French part)"),
                            ("PM", "Saint Pierre and Miquelon"),
                            ("VC", "Saint Vincent and the Grenadines"),
                            ("WS", "Samoa"),
                            ("SM", "San Marino"),
                            ("ST", "Sao Tome and Principe"),
                            ("SA", "Saudi Arabia"),
                            ("SN", "Senegal"),
                            ("RS", "Serbia"),
                            ("SC", "Seychelles"),
                            ("SL", "Sierra Leone"),
                            ("SG", "Singapore"),
                            ("SX", "Sint Maarten (Dutch part)"),
                            ("SK", "Slovakia"),
                            ("SI", "Slovenia"),
                            ("SB", "Solomon Islands"),
                            ("SO", "Somalia"),
                            ("ZA", "South Africa"),
                            ("GS", "South Georgia and the South Sandwich Islands"),
                            ("SS", "South Sudan"),
                            ("ES", "Spain"),
                            ("LK", "Sri Lanka"),
                            ("SD", "Sudan"),
                            ("SR", "Suriname"),
                            ("SJ", "Svalbard and Jan Mayen"),
                            ("SE", "Sweden"),
                            ("CH", "Switzerland"),
                            ("SY", "Syrian Arab Republic"),
                            ("TW", "Taiwan, Province of China"),
                            ("TJ", "Tajikistan"),
                            ("TZ", "Tanzania, United Republic of"),
                            ("TH", "Thailand"),
                            ("TL", "Timor-Leste"),
                            ("TG", "Togo"),
                            ("TK", "Tokelau"),
                            ("TO", "Tonga"),
                            ("TT", "Trinidad and Tobago"),
                            ("TN", "Tunisia"),
                            ("TR", "Turkey"),
                            ("TM", "Turkmenistan"),
                            ("TC", "Turks and Caicos Islands"),
                            ("TV", "Tuvalu"),
                            ("UG", "Uganda"),
                            ("UA", "Ukraine"),
                            ("AE", "United Arab Emirates"),
                            ("US", "United States"),
                            ("UM", "United States Minor Outlying Islands"),
                            ("UY", "Uruguay"),
                            ("UZ", "Uzbekistan"),
                            ("VU", "Vanuatu"),
                            ("VE", "Venezuela, Bolivarian Republic of"),
                            ("VN", "Viet Nam"),
                            ("VG", "Virgin Islands, British"),
                            ("VI", "Virgin Islands, U.S."),
                            ("WF", "Wallis and Futuna"),
                            ("EH", "Western Sahara"),
                            ("YE", "Yemen"),
                            ("ZM", "Zambia"),
                            ("ZW", "Zimbabwe"),
                            ("AX", "Åland Islands"),
                        ],
                        max_length=255,
                        verbose_name="Country",
                    ),
                ),
                (
                    "on_behalf_of_organisation",
                    models.BooleanField(verbose_name="On behalf of organisation"),
                ),
                ("in_memory", models.BooleanField(verbose_name="In memory")),
                (
                    "in_memory_of",
                    models.CharField(
                        blank=True, max_length=60, verbose_name="In memory of"
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="Created at"
                    ),
                ),
                (
                    "donation_page",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailcore.Page",
                    ),
                ),
            ],
            options={
                "permissions": (("can_export_pay_ins", "Can export pay ins"),),
                "abstract": False,
            },
        ),
    ]



# Generated by Django 2.2.11 on 2020-03-30 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0011_donation_rename_address_lines"),
    ]

    operations = [
        migrations.AddField(
            model_name="donation",
            name="provider_transaction_status",
            field=models.CharField(default="", max_length=100, blank=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="donation",
            name="transaction_status",
            field=models.CharField(
                choices=[
                    ("settling", "Settling"),
                    ("settled", "Settled"),
                    ("failed", "Failed"),
                    ("unknown", "Unknown"),
                ],
                max_length=100,
            ),
        ),
    ]



from django.utils.module_loading import import_string

from wagtaildonate.address_lookups.base import AddressLookup
from wagtaildonate.settings import donate_settings


def instantiate_address_lookup(setting, klass=None) -> AddressLookup:
    if klass is None:
        klass = import_string(setting["class"])
    return klass(setting.get("options", {}))


def get_address_lookup() -> AddressLookup:
    return instantiate_address_lookup(donate_settings.ADDRESS_LOOKUP)



from wagtail import blocks
from wagtail.documents.blocks import DocumentChooserBlock
from wagtail.embeds.blocks import EmbedBlock
from wagtail.images.blocks import ImageChooserBlock
from wagtail.snippets.blocks import SnippetChooserBlock


class ImageBlock(blocks.StructBlock):
    image = ImageChooserBlock()
    caption = blocks.CharBlock(required=False)

    class Meta:
        icon = "image"
        template = "wagtaildonate/streamfield/blocks/image_block.html"


class DocumentBlock(blocks.StructBlock):
    document = DocumentChooserBlock()
    title = blocks.CharBlock(required=False)

    class Meta:
        icon = "doc-full-inverse"
        template = "wagtaildonate/streamfield/blocks/document_block.html"


class QuoteBlock(blocks.StructBlock):
    quote = blocks.CharBlock(classname="title")
    attribution = blocks.CharBlock(required=False)

    class Meta:
        icon = "openquote"
        template = "wagtaildonate/streamfield/blocks/quote_block.html"


class ThankYouBlock(blocks.StreamBlock):
    heading = blocks.CharBlock(
        classname="full title",
        icon="title",
        template="wagtaildonate/streamfield/blocks/heading_block.html",
    )
    paragraph = blocks.RichTextBlock()
    image = ImageBlock()
    document = DocumentBlock()
    quote = QuoteBlock()
    video = EmbedBlock()
    call_to_action = SnippetChooserBlock(
        "wagtaildonate.ThankYouCTASnippet",
        template="wagtaildonate/streamfield/blocks/thank_you_cta_block.html",
    )



from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _


class DonationExportLog(models.Model):
    DONATION_TYPE_SINGLE = "single"
    DONATION_TYPE_RECURRING = "recurring"
    DONATION_TYPE_PAY_IN = "pay_in"
    DONATION_TYPE_CHOICES = [
        (DONATION_TYPE_SINGLE, _("Single")),
        (DONATION_TYPE_RECURRING, _("Recurring")),
        (DONATION_TYPE_PAY_IN, _("Pay in")),
    ]

    donation_type = models.CharField(max_length=10, choices=DONATION_TYPE_CHOICES)

    datetime_from = models.DateTimeField()
    datetime_to = models.DateTimeField()
    user = models.ForeignKey(settings.AUTH_USER_MODEL, models.SET_NULL, null=True)
    # In case the user gets deleted.
    user_name = models.CharField(blank=True, max_length=255)
    created_at = models.DateTimeField(default=timezone.now)
    remote_addr = models.CharField(
        blank=True, max_length=255, verbose_name=_("REMOTE_ADDR")
    )
    remote_host = models.CharField(
        blank=True, max_length=255, verbose_name=_("REMOTE_HOST")
    )
    x_forwarded_for = models.CharField(
        blank=True, max_length=255, verbose_name=_("X-Forwarded-For")
    )
    user_agent = models.CharField(blank=True, max_length=255)

    class Meta:
        default_permissions = ["view"]

    def __str__(self):
        return gettext("DonationExportLog object")



import logging

import gocardless_pro

from wagtaildonate.api.serializers.gocardless import GoCardlessDonationSerializer
from wagtaildonate.payment_methods.base import PaymentMethod

logger = logging.getLogger(__name__)


class GoCardlessPaymentMethod(PaymentMethod):
    code = "gocardless"
    serializer_classes = {
        "monthly": GoCardlessDonationSerializer,
    }

    def get_client(self):
        return gocardless_pro.Client(
            access_token=self.options["ACCESS_TOKEN"],
            environment=self.options.get("ENVIRONMENT", "live"),
        )



# Generated by Django 2.2.23 on 2021-06-16 14:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0032_payin_events"),
    ]

    operations = [
        migrations.AddField(
            model_name="payin",
            name="liability_shift_possible",
            field=models.BooleanField(blank=True, default=False),
        ),
        migrations.AddField(
            model_name="payin",
            name="liability_shifted",
            field=models.BooleanField(blank=True, default=False),
        ),
        migrations.AddField(
            model_name="payin",
            name="three_d_secure_status",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
        migrations.AddField(
            model_name="singledonation",
            name="liability_shift_possible",
            field=models.BooleanField(blank=True, default=False),
        ),
        migrations.AddField(
            model_name="singledonation",
            name="liability_shifted",
            field=models.BooleanField(blank=True, default=False),
        ),
        migrations.AddField(
            model_name="singledonation",
            name="three_d_secure_status",
            field=models.CharField(blank=True, default="", max_length=100),
        ),
    ]



# Generated by Django 2.2.7 on 2020-03-04 10:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0005_add_contact_details_fields"),
    ]

    operations = [
        migrations.AddField(
            model_name="donation",
            name="amount",
            field=models.DecimalField(decimal_places=2, default=0.0, max_digits=10),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.2.12 on 2020-05-27 09:30

from django.db import migrations


def delete_change_permissions(apps, schema_editor):
    Permission = apps.get_model("auth", "Permission")
    ContentType = apps.get_model("contenttypes.ContentType")
    db_alias = schema_editor.connection.alias
    try:
        content_type = ContentType.objects.using(db_alias).get(
            model="donationexportlog", app_label="wagtaildonate"
        )
    except ContentType.DoesNotExist:
        pass
    else:
        Permission.objects.using(db_alias).filter(
            content_type=content_type,
            codename__in=[
                "add_donationexportlog",
                "delete_donationexportlog",
                "change_donationexportlog",
            ],
        ).delete()


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0023_optins"),
        ("auth", "0009_alter_user_last_name_max_length"),
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="donationexportlog", options={"default_permissions": ["view"]}
        ),
        migrations.RunPython(delete_change_permissions, migrations.RunPython.noop),
    ]



from django.conf.urls import include
from django.urls import re_path
from django.utils.translation import gettext_lazy as _

from wagtail import hooks
from wagtail.admin.menu import Menu, SubmenuMenuItem

from wagtaildonate import admin_urls
from wagtaildonate.settings import donate_settings


def register_admin_urls():
    return [re_path(r"^donate/", include(admin_urls, namespace="wagtaildonate_admin"))]


class PaymentsMenu(SubmenuMenuItem):
    def is_shown(self, request):
        payment_perms = [
            "wagtaildonate.can_export_donations",
            "wagtaildonate.can_export_recurring_donations",
            "wagtaildonate.can_export_pay_ins",
        ]
        return any(request.user.has_perm(perm) for perm in payment_perms)


def register_payments_menu():
    return PaymentsMenu(
        _("Payments"),
        Menu(register_hook_name="register_payments_menu_item"),
        icon_name="cogs",
    )


def register_general_wagtail_hooks():
    if donate_settings.DONATION_EXPORT_ENABLED:
        hooks.register("register_admin_urls", register_admin_urls)
        hooks.register("register_admin_menu_item", register_payments_menu)



from django.db import models
from django.utils.translation import gettext_lazy as _

from wagtail.admin.panels import FieldPanel


class AbstractPayInEvent(models.Model):
    event_code = models.CharField(max_length=50, unique=True)
    event_name = models.CharField(max_length=255)
    fundraiser_reference_required = models.BooleanField()

    panels = [
        FieldPanel("event_code"),
        FieldPanel("event_name"),
        FieldPanel("fundraiser_reference_required"),
    ]

    class Meta:
        abstract = True
        verbose_name = _("Pay In Event")
        verbose_name_plural = _("Pay In Events")

    def __str__(self):
        return self.event_code


class PayInEvent(AbstractPayInEvent):
    pass



from django.urls import include, path

from wagtaildonate.api import urls as api_urls
from wagtaildonate.views import DonateCheckoutView

app_name = "wagtaildonate"

urlpatterns = [
    path("", DonateCheckoutView.as_view(), name="donate-checkout"),
    path("api/", include(api_urls, namespace="api")),
]



import logging
import re

from django.utils.translation import gettext as _

import braintree
from rest_framework import serializers

from wagtaildonate.api.serializers.utils import (
    get_pay_in_serializer_class,
    get_single_donation_serializer_class,
)
from wagtaildonate.utils.braintree import (
    filter_country_errors,
    filter_postal_code_errors,
    filter_street_address_errors,
    filter_user_card_errors,
    map_status,
)

logger = logging.getLogger(__name__)
SingleDonationSerializer = get_single_donation_serializer_class()
PayInSerializer = get_pay_in_serializer_class()


class BaseBraintreeDonationSerializer(serializers.Serializer):
    payment_method_nonce = serializers.CharField(max_length=255)
    device_data = serializers.CharField(required=False)

    class Meta:
        fields = [
            "device_data",
            "payment_method_nonce",
        ]

    def build_billing_data(self):
        """
        Get billing data ready to be added to the transaction.
        """
        billing_data = {}
        if self.context["payment_method"].should_send_name():
            billing_data["first_name"] = self.validated_data["first_name"]
            billing_data["last_name"] = self.validated_data["surname"]
        if self.context["payment_method"].should_send_postal_code():
            billing_data["postal_code"] = self.validated_data["postal_code"]
        if self.context["payment_method"].should_send_country_code():
            billing_data["country_code_alpha2"] = self.validated_data["country"]
        if self.context["payment_method"].should_send_street_address():
            billing_data["street_address"] = ",".join(
                [
                    self.validated_data["address_line_1"],
                    self.validated_data["address_line_2"],
                ]
            )
            billing_data["extended_address"] = self.validated_data["address_line_3"]
        if self.context["payment_method"].should_send_town():
            billing_data["locality"] = self.validated_data["town"]
        return billing_data

    def build_customer_data(self):
        customer_data = {}
        if self.context["payment_method"].should_send_name():
            customer_data["first_name"] = self.validated_data["first_name"]
            customer_data["last_name"] = self.validated_data["surname"]
        if self.context["payment_method"].should_send_email_address():
            customer_data["email"] = self.validated_data["email"]
        phone_number = self.validated_data["phone_number"]
        if self.context["payment_method"].should_send_phone_number() and phone_number:
            # If we use the PhoneNumber object.
            if hasattr(phone_number, "as_e164"):
                # Delete + as Braintree does not support it.
                customer_data["phone"] = phone_number.as_e164.lstrip("+")
            # If this is returned as a string, delete all non-digit characters.
            elif isinstance(phone_number, str):
                customer_data["phone"] = re.sub("[^0-9]", "", phone_number)
        return customer_data

    def build_transaction_data(self):
        """
        Build transaction data dictionary to be sent to Braintree.
        """
        # Basic transaction data.
        transaction_data = {
            "amount": self.validated_data["amount"],
            "payment_method_nonce": self.validated_data["payment_method_nonce"],
            "options": {"submit_for_settlement": True},
        }
        # Device data
        if self.validated_data.get("device_data"):
            transaction_data["device_data"] = self.validated_data["device_data"]
        # Merchant account ID
        merchant_account_id = self.context["payment_method"].get_merchant_account_id()
        if merchant_account_id:
            transaction_data["merchant_account_id"] = merchant_account_id
        # Billing data (AVS)
        billing_data = self.build_billing_data()
        if billing_data:
            transaction_data["billing"] = billing_data
        customer_data = self.build_customer_data()
        if customer_data:
            transaction_data["customer"] = customer_data
        return transaction_data

    @property
    def gateway(self):
        return self.context["payment_method"].get_gateway()

    def perform_transaction(self):
        try:
            return self.gateway.transaction.sale(self.build_transaction_data())
        except (
            TimeoutError,
            braintree.exceptions.service_unavailable_error.ServiceUnavailableError,
        ) as exception:
            logger.exception("Braintree server timed out")
            raise serializers.ValidationError(
                {
                    "payment_method_nonce": [
                        _("Connection to the server timed out. Please try again later.")
                    ]
                }
            ) from exception
        except (
            braintree.exceptions.too_many_requests_error.TooManyRequestsError
        ) as exception:
            logger.exception("Too many requests to the Braintree account.")
            raise serializers.ValidationError(
                {
                    "payment_method_nonce": [
                        _("Connection to the server timed out. Please try again later.")
                    ]
                }
            ) from exception
        except (
            braintree.exceptions.server_error.ServerError,
            braintree.exceptions.unexpected_error.UnexpectedError,
            braintree.exceptions.upgrade_required_error.UpgradeRequiredError,
            braintree.exceptions.authorization_error.AuthorizationError,
            braintree.exceptions.authentication_error.AuthenticationError,
        ) as exception:
            logger.exception("Unexpected Braintree error.")
            raise serializers.ValidationError(
                {
                    "payment_method_nonce": [
                        _("Unexpected error occurred. Please try again later.")
                    ]
                }
            ) from exception

    def handle_unsuccessful_transaction(self, result):
        logger.info(
            "Braintree transaction unsuccessful: %s - %r",
            result.message,
            result.errors.deep_errors,
        )
        error_dict = {}
        # Card errors
        card_errors = filter_user_card_errors(result)
        if card_errors:
            error_dict["payment_method_nonce"] = card_errors
        # Postal code errors
        postal_code_errors = filter_postal_code_errors(result)
        if postal_code_errors:
            error_dict["postal_code"] = postal_code_errors
        # Country errors
        country_errors = filter_country_errors(result)
        if country_errors:
            error_dict["country"] = country_errors
        # Street address error.
        street_address_errors = filter_street_address_errors(result)
        if street_address_errors:
            error_dict["address_line_1"] = street_address_errors
        # If there's no errors but the transaction is unsuccessful, return a
        # generic error message.
        if not any(error_dict.values()):
            error_dict["payment_method_nonce"] = [
                _(
                    "Sorry there was an error processing your payment. "
                    "Please try again later or use a different card."
                )
            ]
        raise serializers.ValidationError(error_dict)

    def create(self, validated_data):
        result = self.perform_transaction()
        if not result.is_success:
            return self.handle_unsuccessful_transaction(result)

        # Delete non-database fields from the serializer so they are not saved
        # to the donation object in the database which would result in in
        # exception.
        validated_data.pop("payment_method_nonce")
        validated_data.pop("device_data", None)

        # Store Braintree transaction ID/status on the model.
        validated_data["transaction_id"] = result.transaction.id
        validated_data["transaction_status"] = map_status(result.transaction.status)
        validated_data["provider_transaction_status"] = result.transaction.status

        # Store 3D Secure info if it exists
        if result.transaction.three_d_secure_info:
            three_d_secure_info = result.transaction.three_d_secure_info
            validated_data["liability_shifted"] = three_d_secure_info.liability_shifted
            validated_data["liability_shift_possible"] = (
                three_d_secure_info.liability_shift_possible
            )
            validated_data["three_d_secure_status"] = three_d_secure_info.status

        return super().create(validated_data)


class BraintreeSingleDonationSerializer(
    BaseBraintreeDonationSerializer, SingleDonationSerializer  # noqa
):
    class Meta(BaseBraintreeDonationSerializer.Meta, SingleDonationSerializer.Meta):
        fields = (
            SingleDonationSerializer.Meta.fields
            + BaseBraintreeDonationSerializer.Meta.fields
        )


class BraintreePayInSerializer(
    BaseBraintreeDonationSerializer, PayInSerializer  # noqa
):
    class Meta(BaseBraintreeDonationSerializer.Meta, PayInSerializer.Meta):
        fields = (
            PayInSerializer.Meta.fields + BaseBraintreeDonationSerializer.Meta.fields
        )



from wagtaildonate.wagtail_hooks.export import register_export_wagtail_hooks
from wagtaildonate.wagtail_hooks.export_log import register_export_log_wagtail_hooks
from wagtaildonate.wagtail_hooks.general import register_general_wagtail_hooks
from wagtaildonate.wagtail_hooks.pay_in_events import (
    register_pay_in_events_wagtail_hooks,
)

register_export_wagtail_hooks()
register_general_wagtail_hooks()
register_export_log_wagtail_hooks()
register_pay_in_events_wagtail_hooks()



from wagtail_modeladmin.options import ModelAdmin, modeladmin_register

from wagtaildonate.models.utils import get_pay_in_event_model
from wagtaildonate.settings import donate_settings


class PayInEventAdmin(ModelAdmin):
    model = get_pay_in_event_model()

    menu_icon = "doc-full-inverse"
    menu_order = 300
    add_to_settings_menu = False
    exclude_from_explorer = False
    list_display = ("event_name", "event_code")


def register_pay_in_events_wagtail_hooks():
    if donate_settings.PAY_IN_EVENTS_ENABLED:
        modeladmin_register(PayInEventAdmin)



from django.core.exceptions import ImproperlyConfigured
from django.utils.module_loading import import_string

from wagtaildonate.settings import donate_settings


def get_default_donation_page():
    custom_func_path = donate_settings.GET_DEFAULT_DONATION_PAGE_INSTANCE_CALLABLE
    if custom_func_path is not None:
        custom_func = import_string(custom_func_path)
        return custom_func()

    from wagtail.models import Page

    from wagtaildonate.models import AbstractDonationPage

    try:
        return Page.objects.type(AbstractDonationPage).specific().live()[0]
    except IndexError as exception:
        raise ImproperlyConfigured(
            "Default donation landing page is not created."
        ) from exception


def get_single_donation_model_string():
    from wagtaildonate.settings import donate_settings

    return donate_settings.SINGLE_DONATION_MODEL


def get_single_donation_model():
    from django.apps import apps

    return apps.get_model(get_single_donation_model_string())


def get_recurring_donation_model_string():
    from wagtaildonate.settings import donate_settings

    return donate_settings.RECURRING_DONATION_MODEL


def get_recurring_donation_model():
    from django.apps import apps

    return apps.get_model(get_recurring_donation_model_string())


def get_pay_in_model_string():
    from wagtaildonate.settings import donate_settings

    return donate_settings.PAY_IN_MODEL


def get_pay_in_model():
    from django.apps import apps

    return apps.get_model(get_pay_in_model_string())


def get_pay_in_event_model_string():
    from wagtaildonate.settings import donate_settings

    return donate_settings.PAY_IN_EVENT_MODEL


def get_pay_in_event_model():
    from django.apps import apps

    return apps.get_model(get_pay_in_event_model_string())


def get_pay_in_event_serializer_class():
    from wagtaildonate.settings import donate_settings

    return donate_settings.PAY_IN_EVENT_SERIALIZER_CLASS


def get_donation_export_log_model_string():
    from wagtaildonate.settings import donate_settings

    return donate_settings.DONATION_EXPORT_LOG_MODEL


def get_donation_export_log_model():
    from django.apps import apps

    return apps.get_model(get_donation_export_log_model_string())



from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from wagtail import hooks
from wagtail.admin.menu import MenuItem

from wagtaildonate.models.utils import (
    get_pay_in_model,
    get_recurring_donation_model,
    get_single_donation_model,
)
from wagtaildonate.settings import donate_settings


def register_export_permissions():
    donation_model = get_single_donation_model()
    recurring_donation_model = get_recurring_donation_model()
    pay_in_model = get_pay_in_model()
    content_types = ContentType.objects.get_for_models(
        donation_model, recurring_donation_model, pay_in_model
    )
    return Permission.objects.filter(
        Q(content_type=content_types[donation_model], codename="can_export_donations")
        | Q(
            content_type=content_types[recurring_donation_model],
            codename="can_export_recurring_donations",
        )
        | Q(content_type=content_types[pay_in_model], codename="can_export_pay_ins")
    )


class ExportDonationsMenuItem(MenuItem):
    def is_shown(self, request):
        return request.user.has_perm("wagtaildonate.can_export_donations")


def register_export_donations_menu_item():
    return ExportDonationsMenuItem(
        _("Export donations"),
        reverse("wagtaildonate_admin:export"),
        order=1050,
        icon_name="download",
    )


class ExportRecurringDonationsMenuItem(MenuItem):
    def is_shown(self, request):
        return request.user.has_perm("wagtaildonate.can_export_recurring_donations")


def register_export_recurring_donations_menu_item():
    return ExportRecurringDonationsMenuItem(
        _("Export recurring donations"),
        reverse("wagtaildonate_admin:export_recurring"),
        order=1051,
        icon_name="download",
    )


class ExportPayInsMenuItem(MenuItem):
    def is_shown(self, request):
        return request.user.has_perm("wagtaildonate.can_export_pay_ins")


def register_export_pay_ins_menu_item():
    return ExportPayInsMenuItem(
        _("Export pay ins"),
        reverse("wagtaildonate_admin:export_pay_ins"),
        order=1052,
        icon_name="download",
    )


def register_export_wagtail_hooks():
    if donate_settings.DONATION_EXPORT_ENABLED:
        hooks.register("register_permissions", register_export_permissions)
        hooks.register(
            "register_payments_menu_item", register_export_donations_menu_item
        )
        hooks.register(
            "register_payments_menu_item", register_export_recurring_donations_menu_item
        )
        hooks.register("register_payments_menu_item", register_export_pay_ins_menu_item)



# Generated by Django 2.2.7 on 2020-02-21 11:59

import django.db.models.deletion
from django.db import migrations, models

from wagtail.images import get_image_model_string

image_model = get_image_model_string()


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(get_image_model_string()),
        ("wagtaildonate", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="donationfundraisingpayinpage",
            name="hero_image",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to=image_model,
            ),
        ),
        migrations.AddField(
            model_name="donationfundraisingpayinpage",
            name="hero_summary_text",
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name="donationfundraisingpayinpage",
            name="introduction",
            field=models.TextField(blank=True),
        ),
    ]



# Generated by Django 3.2.12 on 2022-05-12 08:25

from django.db import migrations


from wagtail import blocks as wagtail_blocks
from wagtail import fields as wagtail_fields


class Migration(migrations.Migration):
    dependencies = [
        ("wagtaildonate", "0033_three_d_secure_fields"),
    ]

    operations = [
        migrations.AlterField(
            model_name="thankyouctasnippet",
            name="link",
            field=wagtail_fields.StreamField(
                [
                    (
                        "external_link",
                        wagtail_blocks.StructBlock(
                            [
                                ("url", wagtail_blocks.URLBlock()),
                                ("title", wagtail_blocks.CharBlock()),
                            ],
                            icon="link",
                        ),
                    ),
                    (
                        "internal_link",
                        wagtail_blocks.StructBlock(
                            [
                                ("page", wagtail_blocks.PageChooserBlock()),
                                ("title", wagtail_blocks.CharBlock(required=False)),
                            ],
                            icon="link",
                        ),
                    ),
                ],
                blank=True,
                use_json_field=True,
            ),
        ),
    ]



from wagtaildonate.address_lookups.utils import get_address_lookup
from wagtaildonate.countries import get_countries
from wagtaildonate.models.utils import get_pay_in_event_model
from wagtaildonate.payment_methods.utils import get_all_payment_methods
from wagtaildonate.settings import donate_settings


class Configuration:
    def __init__(
        self,
        *,
        checkout_url,
        pay_in_page_id,
        pay_in_success_url,
        allowed_frequencies=None,
        payment_method_context=None
    ):
        self.allowed_frequencies = allowed_frequencies
        self.payment_method_context = payment_method_context
        self.countries = self.get_countries()
        self.checkout_url = checkout_url
        self.payment_methods = self.get_payment_methods()
        self.address_lookup = self.get_address_lookup()
        self.pay_in_events = self.get_pay_in_events()
        self.pay_in_page_id = pay_in_page_id
        self.pay_in_success_url = pay_in_success_url

    def get_payment_methods(self):
        return list(
            get_all_payment_methods(
                allowed_frequencies=self.allowed_frequencies,
                payment_method_context=self.payment_method_context,
            )
        )

    def get_address_lookup(self):
        return get_address_lookup()

    def get_countries(self):
        return tuple(get_countries())

    def get_pay_in_events(self):
        if not donate_settings.PAY_IN_EVENTS_ENABLED:
            return []
        pay_in_event_model = get_pay_in_event_model()
        pay_in_events = list(pay_in_event_model.objects.all())
        if donate_settings.PAY_IN_OTHER_EVENT_ENABLED:
            pay_in_events.append(
                pay_in_event_model(
                    event_code="other",
                    event_name="Other",
                    fundraiser_reference_required=donate_settings.PAY_IN_OTHER_EVENT_REQUIRE_FUNDRAISER_REFERENCE,
                )
            )
        return pay_in_events


def get_configuration_class_string():
    from wagtaildonate.settings import donate_settings

    return donate_settings.CONFIGURATION_CLASS


def get_configuration_class():
    from django.utils.module_loading import import_string

    return import_string(get_configuration_class_string())



# Generated by Django 2.2.12 on 2020-04-24 15:07

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0018_phonenumber_not_mandatory"),
    ]

    operations = [
        migrations.AlterField(
            model_name="donation",
            name="created_at",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Created at"
            ),
        ),
        migrations.AlterField(
            model_name="donationexportlog",
            name="created_at",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="recurringdonation",
            name="created_at",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Created at"
            ),
        ),
    ]



# Generated by Django 2.2.11 on 2020-04-09 14:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0045_assign_unlock_grouppagepermission"),
        ("wagtaildonate", "0016_thank_you_emails"),
    ]

    operations = [
        migrations.AddField(
            model_name="donation",
            name="donation_page",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="wagtailcore.Page",
            ),
        ),
        migrations.AddField(
            model_name="recurringdonation",
            name="donation_page",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="wagtailcore.Page",
            ),
        ),
    ]



from django.utils.translation import gettext as _

import braintree
from braintree import ErrorCodes

from wagtaildonate.models.utils import get_single_donation_model


def filter_user_card_errors(result):
    client_errors = {
        ErrorCodes.CreditCard.CreditCardTypeIsNotAccepted: _(
            "The type of card you used is not accepted."
        ),
        ErrorCodes.Transaction.PaymentMethodNonceCardTypeIsNotAccepted: _(
            "The type of card you used is not accepted."
        ),
        ErrorCodes.CreditCard.CvvIsInvalid: _("The CVV code you entered was invalid."),
        ErrorCodes.CreditCard.CvvIsRequired: _("The CVV code is required."),
        ErrorCodes.CreditCard.CvvVerificationFailed: _(
            "The CVV code you entered was invalid."
        ),
        ErrorCodes.CreditCard.ExpirationDateIsRequired: _(
            "Expiration date is required"
        ),
        ErrorCodes.CreditCard.ExpirationDateIsInvalid: _(
            "The expiration date you entered was invalid."
        ),
        ErrorCodes.CreditCard.NumberIsInvalid: _(
            "The credit card number you entered was invalid."
        ),
        ErrorCodes.ApplePay.ApplePayCardsAreNotAccepted: _(
            "Apple Pay cards are not accepted."
        ),
        ErrorCodes.ApplePay.PaymentMethodNonceCardTypeIsNotAccepted: _(
            "The type of card you used is not accepted."
        ),
    }
    error_messages = [
        client_errors[error.code]
        for error in result.errors.deep_errors
        if error.code in client_errors.keys()
    ]
    return error_messages


def filter_postal_code_errors(result):
    client_errors = {
        ErrorCodes.Address.PostalCodeInvalidCharacters: _(
            "The postal code you provided contains invalid characters."
        ),
        ErrorCodes.Address.PostalCodeIsTooLong: _(
            "The postal code you provided is too long."
        ),
        ErrorCodes.Address.PostalCodeIsInvalid: _(
            "The postal code you provided is not valid."
        ),
        ErrorCodes.CreditCard.PostalCodeVerificationFailed: _(
            "The postal code provided does not match the card's."
        ),
    }
    return list(
        frozenset(
            [
                client_errors[error.code]
                for error in result.errors.deep_errors
                if error.code in client_errors
            ]
        )
    )


def filter_street_address_errors(result):
    client_errors = {
        ErrorCodes.Address.StreetAddressIsInvalid: _(
            "The street address you provided is invalid."
        ),
        ErrorCodes.Address.StreetAddressIsTooLong: _(
            "The street address you provided is too long."
        ),
    }
    return list(
        frozenset(
            [
                client_errors[error.code]
                for error in result.errors.deep_errors
                if error.code in client_errors
            ]
        )
    )


def filter_country_errors(result):
    client_errors = {
        ErrorCodes.Address.CountryCodeAlpha2IsNotAccepted: _(
            "Country provided is not supported."
        ),
        ErrorCodes.Address.CountryCodeAlpha3IsNotAccepted: _(
            "Country provided is not supported."
        ),
        ErrorCodes.Address.CountryCodeNumericIsNotAccepted: _(
            "Country provided is not supported."
        ),
        ErrorCodes.Address.CountryNameIsNotAccepted: _(
            "Country provided is not supported."
        ),
    }
    return list(
        frozenset(
            [
                client_errors[error.code]
                for error in result.errors.deep_errors
                if error.code in client_errors
            ]
        )
    )


Status = braintree.Transaction.Status


def map_status(braintree_status):
    """
    Maps the given braintree status into one of the values in get_single_donation_model().STATUS_CHOICES.
    """
    if braintree_status == Status.Settled:
        return get_single_donation_model().STATUS_SETTLED

    elif braintree_status in {
        Status.Settling,
        Status.SubmittedForSettlement,
        Status.SettlementPending,
    }:
        return get_single_donation_model().STATUS_SETTLING

    elif braintree_status in {
        Status.Failed,
        Status.ProcessorDeclined,
        Status.GatewayRejected,
        Status.Voided,
        Status.SettlementDeclined,
    }:
        return get_single_donation_model().STATUS_FAILED

    else:
        return get_single_donation_model().STATUS_UNKNOWN



# Generated by Django 2.2.12 on 2020-05-07 16:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0019_create_at_to_use_default_timezone_now"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="donation",
            options={
                "permissions": (
                    ("can_export_donations", "Can export donate transactions"),
                ),
                "verbose_name": "Donation",
                "verbose_name_plural": "Donations",
            },
        ),
        migrations.AlterModelOptions(
            name="recurringdonation",
            options={
                "permissions": (
                    (
                        "can_export_recurring_donations",
                        "Can export recurring donate transactions",
                    ),
                ),
                "verbose_name": "Recurring Donation",
                "verbose_name_plural": "Recurring Donations",
            },
        ),
    ]



# Generated by Django 2.2.12 on 2020-06-01 17:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0045_assign_unlock_grouppagepermission"),
        ("wagtaildonate", "0030_addressline3"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="donationfundraisingpayinpage", name="hero_image"
        ),
        migrations.RemoveField(
            model_name="donationfundraisingpayinpage", name="page_ptr"
        ),
        migrations.RemoveField(model_name="donationpage", name="checkout_hero_image"),
        migrations.RemoveField(model_name="donationpage", name="hero_image"),
        migrations.RemoveField(model_name="donationpage", name="page_ptr"),
        migrations.RemoveField(
            model_name="suggesteddonationcampaignpageplacementregular", name="page"
        ),
        migrations.RemoveField(
            model_name="suggesteddonationcampaignpageplacementsingle", name="page"
        ),
        migrations.RemoveField(
            model_name="suggesteddonationdonatepageplacementregular", name="page"
        ),
        migrations.RemoveField(
            model_name="suggesteddonationdonatepageplacementsingle", name="page"
        ),
        migrations.DeleteModel(name="DonationCampaignPage"),
        migrations.DeleteModel(name="DonationFundraisingPayInPage"),
        migrations.DeleteModel(name="DonationPage"),
        migrations.DeleteModel(name="SuggestedDonationCampaignPagePlacementRegular"),
        migrations.DeleteModel(name="SuggestedDonationCampaignPagePlacementSingle"),
        migrations.DeleteModel(name="SuggestedDonationDonatePagePlacementRegular"),
        migrations.DeleteModel(name="SuggestedDonationDonatePagePlacementSingle"),
    ]



import logging

from django.utils.translation import gettext_lazy as _

import requests
from rest_framework import serializers

from wagtaildonate.settings import donate_settings

logger = logging.getLogger(__name__)

RECAPTCHA_VERIFY_URL = "https://www.google.com/recaptcha/api/siteverify"


def validate_recaptcha(recaptcha_token):
    try:
        response = requests.post(
            RECAPTCHA_VERIFY_URL,
            data={
                "secret": donate_settings.RECAPTCHA_PRIVATE_KEY,
                "response": recaptcha_token,
            },
        )
    except requests.exceptions.ConnectionError:
        logger.info("Error validating reCAPTCHA token")
        raise serializers.ValidationError(_("Error validating reCAPTCHA token"))
    if not response.ok:
        logger.info("Error validating reCAPTCHA token")
        raise serializers.ValidationError(_("Error validating reCAPTCHA token"))

    response_data = response.json()
    if not response_data.get("success"):
        logger.info("reCAPTCHA token not valid")
        raise serializers.ValidationError(_("reCAPTCHA token not valid"))

    minimum_score = float(donate_settings.RECAPTCHA_MINIMUM_SCORE)
    if response_data.get("score") < minimum_score:
        logger.info(
            "reCAPTCHA token not valid, score below allowed minimum: %f < %f",
            response_data.get("score"),
            minimum_score,
        )
        raise serializers.ValidationError(_("reCAPTCHA token not valid"))


class RecaptchaField(serializers.CharField):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.write_only = True
        self.validators.append(validate_recaptcha)



# Generated by Django 2.2.12 on 2020-05-29 08:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0026_thankyouemailcontent_not_mandatory"),
    ]

    operations = [
        migrations.AlterField(
            model_name="donationcampaignpage",
            name="thank_you_email_subject",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name="donationfundraisingpayinpage",
            name="thank_you_email_subject",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name="donationpage",
            name="thank_you_email_subject",
            field=models.CharField(blank=True, max_length=255),
        ),
    ]



from django.urls import path

from wagtaildonate.admin_views.export import (
    DonationExportView,
    PayInExportView,
    RecurringDonationExportView,
)
from wagtaildonate.settings import donate_settings

app_name = "wagtaildonate"

urlpatterns = []

if donate_settings.DONATION_EXPORT_ENABLED:
    urlpatterns += [
        path("export/single/", DonationExportView.as_view(), name="export"),
        path(
            "export/recurring/",
            RecurringDonationExportView.as_view(),
            name="export_recurring",
        ),
        path("export/payins/", PayInExportView.as_view(), name="export_pay_ins"),
    ]



# Generated by Django 2.2.7 on 2020-03-10 05:13

from django.db import migrations, models

from wagtaildonate.countries import country_choices


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0006_donation_amount"),
    ]

    operations = [
        migrations.AlterField(
            model_name="donation",
            name="country",
            field=models.CharField(
                choices=country_choices(), max_length=255, verbose_name="Country"
            ),
        ),
    ]



# Generated by Django 2.2.12 on 2020-04-24 12:16

from django.db import migrations, models
import phonenumber_field.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0017_donation_donation_page"),
    ]

    operations = [
        migrations.AlterField(
            model_name="donation",
            name="email",
            field=models.EmailField(
                blank=True, max_length=255, verbose_name="Email address"
            ),
        ),
        migrations.AlterField(
            model_name="donation",
            name="phone_number",
            field=phonenumber_field.modelfields.PhoneNumberField(
                blank=True, max_length=128, region=None, verbose_name="Phone number"
            ),
        ),
        migrations.AlterField(
            model_name="recurringdonation",
            name="email",
            field=models.EmailField(
                blank=True, max_length=255, verbose_name="Email address"
            ),
        ),
        migrations.AlterField(
            model_name="recurringdonation",
            name="phone_number",
            field=phonenumber_field.modelfields.PhoneNumberField(
                blank=True, max_length=128, region=None, verbose_name="Phone number"
            ),
        ),
    ]



# Generated by Django 2.2.11 on 2020-04-09 14:37

from django.db import migrations, models

from wagtail import fields as wagtail_fields

import wagtaildonate.models.pages


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0015_recurringdonation"),
    ]

    operations = [
        migrations.AddField(
            model_name="donationcampaignpage",
            name="thank_you_email_content",
            field=wagtail_fields.StreamField(
                [("paragraph", wagtaildonate.models.pages.EmailParagraphBlock())],
                default="",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donationcampaignpage",
            name="thank_you_email_subject",
            field=models.CharField(default="", max_length=255),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donationfundraisingpayinpage",
            name="thank_you_email_content",
            field=wagtail_fields.StreamField(
                [("paragraph", wagtaildonate.models.pages.EmailParagraphBlock())],
                default="",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donationfundraisingpayinpage",
            name="thank_you_email_subject",
            field=models.CharField(default="", max_length=255),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donationpage",
            name="thank_you_email_content",
            field=wagtail_fields.StreamField(
                [("paragraph", wagtaildonate.models.pages.EmailParagraphBlock())],
                default="",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donationpage",
            name="thank_you_email_subject",
            field=models.CharField(default="", max_length=255),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.2.7 on 2020-03-26 15:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0010_donation_payment_method"),
    ]

    operations = [
        migrations.RenameField(
            model_name="donation", old_name="address_line1", new_name="address_line_1"
        ),
        migrations.RenameField(
            model_name="donation", old_name="address_line2", new_name="address_line_2"
        ),
    ]



# Generated by Django 2.2.7 on 2020-02-28 11:03

from django.db import migrations, models

import phonenumber_field.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0004_thank_you_fields"),
    ]

    operations = [
        migrations.AddField(
            model_name="donation",
            name="address_line1",
            field=models.CharField(
                default="", max_length=255, verbose_name="Address line 1"
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donation",
            name="address_line2",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Address line 2"
            ),
        ),
        migrations.AddField(
            model_name="donation",
            name="country",
            field=models.CharField(default="", max_length=255, verbose_name="Country"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donation",
            name="email",
            field=models.EmailField(
                default="", max_length=255, verbose_name="Email address"
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donation",
            name="first_name",
            field=models.CharField(
                default="", max_length=100, verbose_name="First name"
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donation",
            name="phone_number",
            field=phonenumber_field.modelfields.PhoneNumberField(
                default="", max_length=128, region=None, verbose_name="Phone number"
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donation",
            name="postal_code",
            field=models.CharField(
                default="", max_length=50, verbose_name="Postal code"
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donation",
            name="surname",
            field=models.CharField(default="", max_length=100, verbose_name="Surname"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donation",
            name="town",
            field=models.CharField(default="", max_length=255, verbose_name="Town"),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.2.12 on 2020-05-14 09:40

from django.db import migrations, models
import django.db.models.deletion

from wagtail.images import get_image_model_string

image_model = get_image_model_string()


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(get_image_model_string()),
        ("wagtaildonate", "0020_add_export_permissions"),
    ]

    operations = [
        migrations.AddField(
            model_name="donationcampaignpage",
            name="checkout_hero_image",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to=image_model,
            ),
        ),
        migrations.AddField(
            model_name="donationcampaignpage",
            name="checkout_hero_summary_text",
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name="donationpage",
            name="checkout_hero_image",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to=image_model,
            ),
        ),
        migrations.AddField(
            model_name="donationpage",
            name="checkout_hero_summary_text",
            field=models.TextField(blank=True),
        ),
    ]



import logging

from django.core.management.base import BaseCommand
from django.db import transaction as transaction

from wagtaildonate.exceptions import TransactionNotFound
from wagtaildonate.models.utils import get_pay_in_model, get_single_donation_model

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Update settlement status for non-settled transaction"

    def add_arguments(self, parser):
        parser.add_argument("--dry-run", action="store_true")

    def handle(self, *args, **options):
        models = [get_single_donation_model(), get_pay_in_model()]

        dry_run = options["dry_run"]
        if dry_run:
            logger.info(
                "Dry running updating settlement status of settling transactions."
            )
        else:
            logger.info("Updating settlement status of settling transactions.")

        for model in models:
            self.handle_model(model, dry_run=dry_run)

    def handle_model(self, model, dry_run=False):
        queryset = model.objects.settling().values_list("pk", flat=True)
        for donation_pk in queryset.iterator():
            if dry_run:
                donation = model.objects.get(pk=donation_pk)
                try:
                    donation.update_settlement_status(
                        commit=False, output_non_commit_log=True
                    )
                except TransactionNotFound:
                    logger.exception(
                        "Could not load remote transaction data for %s (ID: %s).",
                        type(donation).__name__,
                        donation.pk,
                    )
            else:
                with transaction.atomic():
                    # Lock this donation instance for update so nothing else tries to
                    # update it.
                    donation = model.objects.select_for_update().get(pk=donation_pk)
                    if not donation.is_settling():
                        # Skip this iteration because the transaction is not settling
                        # anymore.
                        continue
                    try:
                        donation.update_settlement_status()
                    except TransactionNotFound:
                        logger.exception(
                            "Could not load remote transaction data for %s (ID: %s).",
                            type(donation).__name__,
                            donation.pk,
                        )



# Generated by Django 2.2.7 on 2020-02-24 18:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0002_add_fundraising_pay_in_fields"),
    ]

    operations = [
        migrations.CreateModel(
            name="Donation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "transaction_id",
                    models.CharField(max_length=50, verbose_name="Transaction ID"),
                ),
                ("transaction_status", models.CharField(blank=True, max_length=100)),
            ],
            options={"verbose_name": "Donation", "verbose_name_plural": "Donations"},
        ),
    ]



from decimal import Decimal

from django.contrib.humanize.templatetags.humanize import intcomma


def format_gbp(amount) -> str:
    # Convert to decimal as Donation.amount could be a string on create
    amount = Decimal(amount)

    pounds = int(amount)
    pence = int((amount - pounds) * 100)
    return f"£{intcomma(pounds)}.{str(pence).zfill(2)}"



from wagtail import hooks

from wagtail_modeladmin.helpers import PermissionHelper
from wagtail_modeladmin.options import ModelAdmin, modeladmin_register

from wagtaildonate.models import get_donation_export_log_model
from wagtaildonate.settings import donate_settings


class ReadOnlyPermissionHelper(PermissionHelper):
    def user_can_edit_obj(self, *args, **kwargs):
        return False

    def user_can_delete_obj(self, *args, **kwargs):
        return False

    def user_can_create(self, *args, **kwargs):
        return False

    def get_all_model_permissions(self, *args, **kwargs):
        return (
            super()
            .get_all_model_permissions(*args, **kwargs)
            .filter(codename__startswith="view_")
        )


class DonationExportLogAdmin(ModelAdmin):
    model = get_donation_export_log_model()
    menu_label = "Export logs"
    menu_order = 1060
    permission_helper_class = ReadOnlyPermissionHelper
    inspect_view_enabled = True
    ordering = ["-created_at"]
    menu_icon = "list-ul"
    list_display = [
        "created_at",
        "donation_type",
        "datetime_from",
        "datetime_to",
        "get_user",
    ]
    list_filter = ["donation_type"]
    search_fields = ["user__pk", "user_name"]

    def get_user(self, obj):
        if obj.user:
            return obj.user
        return obj.user_name

    get_user.short_description = "User"

    # Copied from Wagtail to allow registering a custom menu hook.
    # https://github.com/wagtail/wagtail/blob/314a926f75f96275ccd9c84a568d7e588f13b258/wagtail/contrib/modeladmin/options.py#L30-L56
    # Related issue: https://github.com/wagtail/wagtail/issues/5302
    def register_with_wagtail(self):
        @hooks.register("register_permissions")
        def register_permissions():
            return self.get_permissions_for_registration()

        @hooks.register("register_admin_urls")
        def register_admin_urls():
            return self.get_admin_urls_for_registration()

        @hooks.register("register_payments_menu_item")
        def register_admin_menu_item():
            return self.get_menu_item()

        # Overriding the explorer page queryset is a somewhat 'niche' / experimental
        # operation, so only attach that hook if we specifically opt into it
        # by returning True from will_modify_explorer_page_queryset
        if self.will_modify_explorer_page_queryset():

            @hooks.register("construct_explorer_page_queryset")
            def construct_explorer_page_queryset(parent_page, queryset, request):
                return self.modify_explorer_page_queryset(
                    parent_page, queryset, request
                )


def register_export_log_wagtail_hooks():
    if donate_settings.DONATION_EXPORT_LOG_ENABLED:
        modeladmin_register(DonationExportLogAdmin)



from decimal import Decimal

from django.utils.translation import gettext_lazy as _

from phonenumber_field.serializerfields import PhoneNumberField
from rest_framework import serializers

from wagtaildonate.api.serializers.recaptcha import RecaptchaField
from wagtaildonate.models import BaseDonation
from wagtaildonate.models.utils import (
    get_pay_in_event_model,
    get_pay_in_model,
    get_recurring_donation_model,
    get_single_donation_model,
)
from wagtaildonate.settings import donate_settings

OTHER_EVENT_CODE = "other"


class BaseDonationSerializer(serializers.ModelSerializer):
    """
    Base serializer used for pay in, single and recurring donations.
    """

    phone_number = PhoneNumberField()
    recaptcha_token = RecaptchaField()

    class Meta:
        model = BaseDonation
        fields = [
            "donation_page",
            "payment_method",
            "id",
            "amount",
            "first_name",
            "surname",
            "email",
            "phone_number",
            "address_line_1",
            "address_line_2",
            "address_line_3",
            "town",
            "postal_code",
            "country",
            "on_behalf_of_organisation",
            "in_memory",
            "in_memory_of",
            "sms_consent",
            "phone_consent",
            "email_consent",
            "post_consent",
            "recaptcha_token",
        ]
        extra_kwargs = {
            "id": {"read_only": True},
            "phone_number": {
                "required": donate_settings.PHONE_NUMBER_REQUIRED,
                "allow_blank": not donate_settings.PHONE_NUMBER_REQUIRED,
            },
            "email": {
                "required": donate_settings.EMAIL_REQUIRED,
                "allow_blank": not donate_settings.EMAIL_REQUIRED,
            },
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # If RECAPTCHA_PRIVATE_KEY is not set, don't attempt to validate recaptcha
        if not donate_settings.RECAPTCHA_PRIVATE_KEY:
            self.fields.pop("recaptcha_token")

    def validate_amount(self, value):
        minimum_amount = Decimal(self.context.get("minimum_amount", Decimal("1.00")))
        maximum_amount = Decimal(
            self.context.get("maximum_amount", Decimal("10000.00"))
        )
        if value < minimum_amount:
            raise serializers.ValidationError(_(f"Minimum amount is {minimum_amount}."))
        if value > maximum_amount:
            raise serializers.ValidationError(_(f"Maximum amount is {maximum_amount}."))
        return value

    def validate(self, data):
        data = super().validate(data)
        # recaptcha_token isn't needed past this point
        data.pop("recaptcha_token", "")
        return data


class SingleDonationSerializer(BaseDonationSerializer):
    """
    Base serializer for single donations.
    """

    class Meta(BaseDonationSerializer.Meta):
        model = get_single_donation_model()
        fields = BaseDonationSerializer.Meta.fields + [
            "gift_aid_declaration",
        ]


class RecurringDonationSerializer(BaseDonationSerializer):
    """
    Base serializer for recurring donations.
    """

    class Meta(BaseDonationSerializer.Meta):
        model = get_recurring_donation_model()
        fields = BaseDonationSerializer.Meta.fields + [
            "gift_aid_declaration",
            "frequency",
        ]


class PayInSerializer(BaseDonationSerializer):
    class Meta(BaseDonationSerializer.Meta):
        model = get_pay_in_model()
        fields = BaseDonationSerializer.Meta.fields + [
            "event_name",
            "event_code",
            "fundraiser_reference",
        ]

    def validate(self, data):
        data = super().validate(data)

        if not donate_settings.PAY_IN_EVENTS_ENABLED:
            return data

        fundraiser_reference_required = False

        pay_in_event_model = get_pay_in_event_model()
        if data["event_code"] != OTHER_EVENT_CODE:
            try:
                event = pay_in_event_model.objects.get(event_code=data["event_code"])
            except pay_in_event_model.DoesNotExist:
                raise serializers.ValidationError(
                    {
                        "event_code": [
                            _(
                                "Pay-in event does not exist. Please select a valid event."
                            )
                        ]
                    }
                )
            fundraiser_reference_required = event.fundraiser_reference_required
        else:
            fundraiser_reference_required = (
                donate_settings.PAY_IN_OTHER_EVENT_REQUIRE_FUNDRAISER_REFERENCE
            )

        if fundraiser_reference_required and not data.get("fundraiser_reference"):
            raise serializers.ValidationError(
                {"fundraiser_reference": [_("Fundraiser reference must be entered.")]}
            )

        return data



from datetime import date, datetime, time

from django.contrib.auth.mixins import PermissionRequiredMixin
from django.utils import timezone
from django.utils.translation import gettext as _
from django.views.generic import FormView

from wagtail.admin.views.mixins import SpreadsheetExportMixin

from wagtaildonate.admin_forms import DateRangeForm
from wagtaildonate.models.utils import (
    get_donation_export_log_model,
    get_pay_in_model,
    get_recurring_donation_model,
    get_single_donation_model,
)
from wagtaildonate.settings import donate_settings


def local_day_start(value: date) -> datetime:
    return timezone.localtime(timezone.make_aware(datetime.combine(value, time.min)))


def local_day_end(value: date) -> datetime:
    return timezone.localtime(timezone.make_aware(datetime.combine(value, time.max)))


class BaseExportView(SpreadsheetExportMixin, FormView):
    template_name = "wagtaildonate/admin/export_view.html"
    donation_model = None
    export_log_model = get_donation_export_log_model()
    form_class = DateRangeForm
    log_donation_type = ""
    title = _("Export donations")

    def get_donation_model(self):
        return self.donation_model

    def get_queryset(self):
        return self.get_donation_model().objects.all()

    def get_export_log_model(self):
        return self.export_log_model

    def get_log_donation_type(self):
        return self.log_donation_type

    def get_user_information(self):
        model = self.get_export_log_model()
        remote_addr_max_length = model._meta.get_field("remote_addr").max_length
        remote_host_max_length = model._meta.get_field("remote_host").max_length
        user_name_max_length = model._meta.get_field("user_name").max_length
        x_forwarded_for_max_length = model._meta.get_field("x_forwarded_for").max_length
        user_agent_max_length = model._meta.get_field("user_agent").max_length
        remote_host = self.request.META.get("REMOTE_HOST", "")[:remote_host_max_length]
        remote_addr = self.request.META.get("REMOTE_ADDR", "")[:remote_addr_max_length]
        x_forwarded_for = self.request.META.get("HTTP_X_FORWARDED_FOR", "")[
            :x_forwarded_for_max_length
        ]
        user_agent = self.request.META.get("HTTP_USER_AGENT", "")[
            :user_agent_max_length
        ]
        user_name = self.request.user.get_username()[:user_name_max_length]
        return {
            "remote_addr": remote_addr,
            "remote_host": remote_host,
            "x_forwarded_for": x_forwarded_for,
            "user_agent": user_agent,
            "user": self.request.user,
            "user_name": user_name,
        }

    def form_valid(self, form):
        from_datetime = (
            local_day_start(form.cleaned_data["from_date"])
            if form.cleaned_data["from_date"]
            else None
        )

        to_datetime = (
            local_day_end(form.cleaned_data["to_date"])
            if form.cleaned_data["to_date"]
            else None
        )

        model = self.get_export_log_model()

        # Log
        model.objects.create(
            donation_type=self.get_log_donation_type(),
            datetime_from=from_datetime,
            datetime_to=to_datetime,
            **self.get_user_information(),
        )

        donations = self.get_queryset()

        if from_datetime:
            donations = donations.filter(created_at__gte=from_datetime)

        if to_datetime:
            donations = donations.filter(created_at__lte=to_datetime)

        return self.as_spreadsheet(donations, self.request.POST.get("format", "csv"))

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["title"] = self.title
        return context


class DonationExportView(PermissionRequiredMixin, BaseExportView):
    donation_model = get_single_donation_model()
    list_export = [
        "created_at",
        "payment_method",
        "transaction_id",
        "get_transaction_status_display",
        "provider_transaction_status",
        "amount",
        "first_name",
        "surname",
        "email",
        "phone_number",
        "address_line_1",
        "address_line_2",
        "address_line_3",
        "town",
        "postal_code",
        "country",
        "on_behalf_of_organisation",
        "gift_aid_declaration",
        "in_memory",
        "in_memory_of",
        "sms_consent",
        "phone_consent",
        "email_consent",
        "post_consent",
        "donation_page_id",
    ]
    export_headings = {
        "payment_method": "Provider",
        "donation_page_id": "Donation Page",
        "get_transaction_status_display": "Transaction Status",
        "sms_consent": "SMS Consent",
    }
    log_donation_type = get_donation_export_log_model().DONATION_TYPE_SINGLE
    permission_required = "wagtaildonate.can_export_donations"

    def __init__(self):
        if donate_settings.THREE_D_SECURE_EXPORT_FIELDS:
            self.list_export += [
                "liability_shifted",
                "liability_shift_possible",
                "three_d_secure_status",
            ]
            self.export_headings = {
                **self.export_headings,
                **{
                    "liability_shifted": "3D Secure Liability Shifted",
                    "liability_shift_possible": "3D Secure Liability Shift Possible",
                    "three_d_secure_status": "3D Secure Status",
                },
            }

    def get_filename(self):
        return "single-donations-export-" + timezone.localdate().isoformat()


class RecurringDonationExportView(PermissionRequiredMixin, BaseExportView):
    donation_model = get_recurring_donation_model()
    list_export = [
        "created_at",
        "payment_method",
        "subscription_id",
        "amount",
        "get_frequency_display",
        "first_name",
        "surname",
        "email",
        "phone_number",
        "address_line_1",
        "address_line_2",
        "address_line_3",
        "town",
        "postal_code",
        "country",
        "on_behalf_of_organisation",
        "gift_aid_declaration",
        "in_memory",
        "in_memory_of",
        "sms_consent",
        "phone_consent",
        "email_consent",
        "post_consent",
        "donation_page_id",
    ]
    export_headings = {
        "payment_method": "Provider",
        "get_frequency_display": "Frequency",
        "sms_consent": "SMS Consent",
        "donation_page_id": "Donation Page",
    }
    log_donation_type = get_donation_export_log_model().DONATION_TYPE_RECURRING
    permission_required = "wagtaildonate.can_export_recurring_donations"
    title = _("Export recurring donations")

    def get_filename(self):
        return "recurring-donations-export-" + timezone.localdate().isoformat()


class PayInExportView(PermissionRequiredMixin, BaseExportView):
    donation_model = get_pay_in_model()
    list_export = [
        "created_at",
        "payment_method",
        "transaction_id",
        "get_transaction_status_display",
        "provider_transaction_status",
        "amount",
        "first_name",
        "surname",
        "email",
        "phone_number",
        "address_line_1",
        "address_line_2",
        "address_line_3",
        "town",
        "postal_code",
        "country",
        "on_behalf_of_organisation",
        "in_memory",
        "in_memory_of",
        "sms_consent",
        "phone_consent",
        "email_consent",
        "post_consent",
        "donation_page_id",
        "event_name",
        "event_code",
        "fundraiser_reference",
    ]
    export_headings = {
        "payment_method": "Provider",
        "donation_page_id": "Donation Page",
        "get_transaction_status_display": "Transaction Status",
        "sms_consent": "SMS Consent",
    }
    log_donation_type = get_donation_export_log_model().DONATION_TYPE_PAY_IN
    permission_required = "wagtaildonate.can_export_pay_ins"

    def __init__(self):
        if donate_settings.THREE_D_SECURE_EXPORT_FIELDS:
            self.list_export += [
                "liability_shifted",
                "liability_shift_possible",
                "three_d_secure_status",
            ]
            self.export_headings = {
                **self.export_headings,
                **{
                    "liability_shifted": "3D Secure Liability Shifted",
                    "liability_shift_possible": "3D Secure Liability Shift Possible",
                    "three_d_secure_status": "3D Secure Status",
                },
            }

    def get_filename(self):
        return "pay-ins-export-" + timezone.localdate().isoformat()



from wagtail import blocks


class DonateBlock(blocks.StructBlock):
    donation_page = blocks.PageChooserBlock()

    class Meta:
        icon = "plus-inverse"
        template = "wagtaildonate/streamfield/blocks/donate_block.html"

    def get_context(self, value, parent_context=None):
        context = super().get_context(value, parent_context=parent_context)

        donation_page = value["donation_page"].specific
        context["donation_page"] = donation_page
        context.update(donation_page.get_donation_values())

        return context



from wagtaildonate.models.donations import (
    BaseDonation,
    PayIn,
    RecurringDonation,
    SingleDonation,
)
from wagtaildonate.models.export import DonationExportLog
from wagtaildonate.models.pages import (
    AbstractDonationCampaignPage,
    AbstractDonationFundraisingPayInPage,
    AbstractDonationLandingPage,
    AbstractDonationPage,
    AbstractPayInLandingPage,
    DonationLandingPageBase,
    SuggestedAmount,
)
from wagtaildonate.models.pay_in_events import AbstractPayInEvent, PayInEvent
from wagtaildonate.models.snippets import ThankYouCTASnippet
from wagtaildonate.models.utils import (
    get_default_donation_page,
    get_donation_export_log_model,
    get_donation_export_log_model_string,
    get_pay_in_event_model_string,
    get_pay_in_model_string,
    get_recurring_donation_model,
    get_recurring_donation_model_string,
    get_single_donation_model,
    get_single_donation_model_string,
)

__all__ = [
    "AbstractPayInLandingPage",
    "AbstractDonationLandingPage",
    "AbstractDonationCampaignPage",
    "AbstractDonationFundraisingPayInPage",
    "AbstractPayInEvent",
    "DonationLandingPageBase",
    "AbstractDonationPage",
    "SuggestedAmount",
    "ThankYouCTASnippet",
    "SingleDonation",
    "RecurringDonation",
    "PayIn",
    "PayInEvent",
    "BaseDonation",
    "DonationExportLog",
    "get_pay_in_model_string",
    "get_pay_in_event_model_string",
    "get_single_donation_model",
    "get_single_donation_model_string",
    "get_recurring_donation_model",
    "get_recurring_donation_model_string",
    "get_default_donation_page",
    "get_donation_export_log_model",
    "get_donation_export_log_model_string",
]



from django.utils.translation import gettext_lazy as _

import pycountry


def country_choices():
    for country in get_countries():
        yield country.alpha_2, _(country.name)


def get_countries():
    prefix_list = []
    countries_list = []
    for country in pycountry.countries:
        # Put the UK and ROI as the first item on the list.
        if country.alpha_2 == "GB":
            prefix_list.insert(0, country)
        elif country.alpha_2 == "IE":
            prefix_list.append(country)
        else:
            countries_list.append(country)
    countries_list.sort(key=lambda c: c.name)
    return prefix_list + countries_list



# Generated by Django 2.2.11 on 2020-04-09 10:40

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("wagtaildonate", "0012_donation_transaction_status_choices"),
    ]

    operations = [
        migrations.CreateModel(
            name="DonationExportLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("datetime_from", models.DateTimeField()),
                ("datetime_to", models.DateTimeField()),
                ("user_name", models.CharField(blank=True, max_length=255)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                (
                    "user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



from django import template

from wagtaildonate.address_lookups.utils import get_address_lookup
from wagtaildonate.payment_methods.utils import get_all_payment_methods
from wagtaildonate.utils.assets import get_source_code_for_assets

register = template.Library()


@register.simple_tag
def payment_methods_assets(frequency=None):
    """
    Load assets in HTML for all active payment methods.
    """
    payment_methods = get_all_payment_methods()
    js_assets = []
    css_assets = []
    for payment_method in payment_methods:
        # Filter by frequency if supplied.
        if frequency is not None and not payment_method.is_frequency_supported(
            frequency
        ):
            continue
        method_assets = payment_method.get_assets()
        method_js_assets = method_assets.get("js", [])
        method_css_assets = method_assets.get("css", [])
        for method_js_asset in method_js_assets:
            # Eliminate duplicates.
            if method_js_asset not in js_assets:
                js_assets.append(method_js_asset)
        for method_css_asset in method_css_assets:
            # Eliminate duplicates.
            if method_css_asset not in css_assets:
                css_assets.append(method_css_assets)
    return get_source_code_for_assets({"js": js_assets, "css": css_assets})


@register.simple_tag
def address_lookups_assets(frequency=None):
    """
    Load assets in HTML for the current address lookup.
    """
    address_lookup = get_address_lookup()
    assets = address_lookup.get_assets()
    return get_source_code_for_assets(assets)



# Generated by Django 2.2.12 on 2020-05-14 23:56

from django.db import migrations, models
import django.db.models.deletion

from wagtail import blocks as wagtail_blocks
from wagtail import fields as wagtail_fields

import wagtail.documents.blocks
import wagtail.embeds.blocks
import wagtail.images.blocks
import wagtail.snippets.blocks

from wagtail.images import get_image_model_string

image_model = get_image_model_string()


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(get_image_model_string()),
        ("wagtaildonate", "0021_checkout_hero"),
    ]

    operations = [
        migrations.AddField(
            model_name="donationcampaignpage",
            name="thank_you_page_content",
            field=wagtail_fields.StreamField(
                [
                    (
                        "heading",
                        wagtail_blocks.CharBlock(
                            classname="full title",
                            icon="title",
                            template="wagtaildonate/streamfield/blocks/heading_block.html",
                        ),
                    ),
                    ("paragraph", wagtail_blocks.RichTextBlock()),
                    (
                        "image",
                        wagtail_blocks.StructBlock(
                            [
                                ("image", wagtail.images.blocks.ImageChooserBlock()),
                                (
                                    "caption",
                                    wagtail_blocks.CharBlock(required=False),
                                ),
                            ]
                        ),
                    ),
                    (
                        "document",
                        wagtail_blocks.StructBlock(
                            [
                                (
                                    "document",
                                    wagtail.documents.blocks.DocumentChooserBlock(),
                                ),
                                (
                                    "title",
                                    wagtail_blocks.CharBlock(required=False),
                                ),
                            ]
                        ),
                    ),
                    (
                        "quote",
                        wagtail_blocks.StructBlock(
                            [
                                (
                                    "quote",
                                    wagtail_blocks.CharBlock(classname="title"),
                                ),
                                (
                                    "attribution",
                                    wagtail_blocks.CharBlock(required=False),
                                ),
                            ]
                        ),
                    ),
                    ("video", wagtail.embeds.blocks.EmbedBlock()),
                    (
                        "call_to_action",
                        wagtail.snippets.blocks.SnippetChooserBlock(
                            "wagtaildonate.ThankYouCTASnippet",
                            template="wagtaildonate/streamfield/blocks/thank_you_cta_block.html",
                        ),
                    ),
                ],
                blank=True,
            ),
        ),
        migrations.AddField(
            model_name="donationfundraisingpayinpage",
            name="thank_you_page_content",
            field=wagtail_fields.StreamField(
                [
                    (
                        "heading",
                        wagtail_blocks.CharBlock(
                            classname="full title",
                            icon="title",
                            template="wagtaildonate/streamfield/blocks/heading_block.html",
                        ),
                    ),
                    ("paragraph", wagtail_blocks.RichTextBlock()),
                    (
                        "image",
                        wagtail_blocks.StructBlock(
                            [
                                ("image", wagtail.images.blocks.ImageChooserBlock()),
                                (
                                    "caption",
                                    wagtail_blocks.CharBlock(required=False),
                                ),
                            ]
                        ),
                    ),
                    (
                        "document",
                        wagtail_blocks.StructBlock(
                            [
                                (
                                    "document",
                                    wagtail.documents.blocks.DocumentChooserBlock(),
                                ),
                                (
                                    "title",
                                    wagtail_blocks.CharBlock(required=False),
                                ),
                            ]
                        ),
                    ),
                    (
                        "quote",
                        wagtail_blocks.StructBlock(
                            [
                                (
                                    "quote",
                                    wagtail_blocks.CharBlock(classname="title"),
                                ),
                                (
                                    "attribution",
                                    wagtail_blocks.CharBlock(required=False),
                                ),
                            ]
                        ),
                    ),
                    ("video", wagtail.embeds.blocks.EmbedBlock()),
                    (
                        "call_to_action",
                        wagtail.snippets.blocks.SnippetChooserBlock(
                            "wagtaildonate.ThankYouCTASnippet",
                            template="wagtaildonate/streamfield/blocks/thank_you_cta_block.html",
                        ),
                    ),
                ],
                blank=True,
            ),
        ),
        migrations.AddField(
            model_name="donationpage",
            name="thank_you_page_content",
            field=wagtail_fields.StreamField(
                [
                    (
                        "heading",
                        wagtail_blocks.CharBlock(
                            classname="full title",
                            icon="title",
                            template="wagtaildonate/streamfield/blocks/heading_block.html",
                        ),
                    ),
                    ("paragraph", wagtail_blocks.RichTextBlock()),
                    (
                        "image",
                        wagtail_blocks.StructBlock(
                            [
                                ("image", wagtail.images.blocks.ImageChooserBlock()),
                                (
                                    "caption",
                                    wagtail_blocks.CharBlock(required=False),
                                ),
                            ]
                        ),
                    ),
                    (
                        "document",
                        wagtail_blocks.StructBlock(
                            [
                                (
                                    "document",
                                    wagtail.documents.blocks.DocumentChooserBlock(),
                                ),
                                (
                                    "title",
                                    wagtail_blocks.CharBlock(required=False),
                                ),
                            ]
                        ),
                    ),
                    (
                        "quote",
                        wagtail_blocks.StructBlock(
                            [
                                (
                                    "quote",
                                    wagtail_blocks.CharBlock(classname="title"),
                                ),
                                (
                                    "attribution",
                                    wagtail_blocks.CharBlock(required=False),
                                ),
                            ]
                        ),
                    ),
                    ("video", wagtail.embeds.blocks.EmbedBlock()),
                    (
                        "call_to_action",
                        wagtail.snippets.blocks.SnippetChooserBlock(
                            "wagtaildonate.ThankYouCTASnippet",
                            template="wagtaildonate/streamfield/blocks/thank_you_cta_block.html",
                        ),
                    ),
                ],
                blank=True,
            ),
        ),
        migrations.CreateModel(
            name="ThankYouCTASnippet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=255)),
                (
                    "summary",
                    wagtail_fields.RichTextField(blank=True, max_length=255),
                ),
                (
                    "link",
                    wagtail_fields.StreamField(
                        [
                            (
                                "external_link",
                                wagtail_blocks.StructBlock(
                                    [
                                        ("url", wagtail_blocks.URLBlock()),
                                        ("title", wagtail_blocks.CharBlock()),
                                    ],
                                    icon="link",
                                ),
                            ),
                            (
                                "internal_link",
                                wagtail_blocks.StructBlock(
                                    [
                                        (
                                            "page",
                                            wagtail_blocks.PageChooserBlock(),
                                        ),
                                        (
                                            "title",
                                            wagtail_blocks.CharBlock(required=False),
                                        ),
                                    ],
                                    icon="link",
                                ),
                            ),
                        ],
                        blank=True,
                    ),
                ),
                (
                    "image",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=image_model,
                    ),
                ),
            ],
            options={"verbose_name": "Thank you CTA snippet"},
        ),
    ]



from typing import Any, ClassVar, Dict, List, Type

from rest_framework import serializers

from wagtaildonate.transaction_status import TransactionStatus

PaymentMethodOptions = Dict[str, Any]


class PaymentMethod:
    FREQUENCY_PAYIN = "payin"
    FREQUENCY_SINGLE = "single"
    FREQUENCY_MONTHLY = "monthly"
    code: ClassVar[str]
    assets: ClassVar[Dict[str, Any]] = None
    serializer_classes: ClassVar[Dict[str, Type[serializers.Serializer]]] = {}
    options: PaymentMethodOptions
    context: Dict[str, Any]

    def __init__(
        self, options: PaymentMethodOptions, *, context: Dict[str, Any] = None
    ):
        self.options = options
        self.context = context

        if self.context is None:
            self.context = {}

    def get_frontend_options(self) -> PaymentMethodOptions:
        return {}

    def get_transaction_status_for_transaction(
        self, transaction_id: str
    ) -> TransactionStatus:
        raise NotImplementedError(
            "get_transaction_status_for_transaction method is not implemented"
        )

    def get_assets(self) -> Dict[str, Any]:
        if self.assets is not None:
            return self.assets.copy()
        return {}

    def get_supported_frequencies(self) -> List[str]:
        return list(self.serializer_classes.keys())

    def is_frequency_supported(self, frequency: str) -> bool:
        return frequency in self.get_supported_frequencies()

    def get_serializer_class_for_frequency(
        self, frequency: str
    ) -> Type[serializers.Serializer]:
        return self.serializer_classes[frequency]
